@@ -90,6 +90,10 @@ xfce_xkb_construct (XfcePanelPlugin *plugin)
     t_xkb *xkb = xkb_new (plugin);
     xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
+#ifdef HAS_PANEL_49
+    xfce_panel_plugin_set_small (plugin, TRUE);
+#endif
+
     g_signal_connect (plugin, "orientation-changed",
             G_CALLBACK (xfce_xkb_orientation_changed), xkb);
 
@@ -168,17 +172,13 @@ xkb_new (XfcePanelPlugin *plugin)
     xkb->settings = g_new0 (t_xkb_settings, 1);
     xkb->plugin = plugin;
 
-    filename = xfce_panel_plugin_save_location (plugin, TRUE);
-    if ((!filename) || (!xkb_load_config (xkb, filename)))
-    {
-        xkb_load_default (xkb);
-    }
-    g_free (filename);
+    xkb_load_default (xkb);
 
     xkb->btn = gtk_button_new ();
     gtk_button_set_relief (GTK_BUTTON (xkb->btn), GTK_RELIEF_NONE);
     gtk_container_add (GTK_CONTAINER (xkb->plugin), xkb->btn);
     xfce_panel_plugin_add_action_widget (xkb->plugin, xkb->btn);
+    xkb->button_state = GTK_STATE_NORMAL;
 
     gtk_widget_show (xkb->btn);
     g_signal_connect (xkb->btn, "clicked", G_CALLBACK (xkb_plugin_button_clicked), xkb);
@@ -209,6 +209,15 @@ xkb_new (XfcePanelPlugin *plugin)
         xkb_initialize_menu (xkb);
     }
 
+    filename = xfce_panel_plugin_lookup_rc_file (plugin);
+    if (xkb_load_config (xkb, filename))
+    {
+        xkb_config_update_settings (xkb->settings);
+    }
+    g_free (filename);
+
+
+
     wnck_screen = wnck_screen_get_default ();
     g_signal_connect (G_OBJECT (wnck_screen), "active-window-changed",
             G_CALLBACK (xkb_plugin_active_window_changed), xkb);
@@ -289,12 +298,15 @@ static gboolean
 xkb_load_config (t_xkb *xkb, const gchar *filename)
 {
     XfceRc* rcfile;
-    if ((rcfile = xfce_rc_simple_open (filename, TRUE)))
+
+    TRACE ("rc filename: %s", filename);
+
+    if (filename && (rcfile = xfce_rc_simple_open (filename, TRUE)))
     {
         xfce_rc_set_group (rcfile, NULL);
 
         xkb->display_type = xfce_rc_read_int_entry (rcfile, "display_type", DISPLAY_TYPE_IMAGE);
-        xkb->display_textsize = xfce_rc_read_int_entry (rcfile, "display_textsize", DISPLAY_TEXTSIZE_SMALL);
+        xkb->display_textsize = xfce_rc_read_int_entry (rcfile, "display_textsize", DISPLAY_TEXTSIZE_LARGE);
         xkb->settings->group_policy = xfce_rc_read_int_entry (rcfile, "group_policy", GROUP_POLICY_PER_APPLICATION);
 
         if (xkb->settings->group_policy != GROUP_POLICY_GLOBAL)
@@ -304,15 +316,12 @@ xkb_load_config (t_xkb *xkb, const gchar *filename)
 
         xkb->settings->never_modify_config = xfce_rc_read_bool_entry (rcfile, "never_modify_config", FALSE);
 
-        if (xkb->settings->kbd_config == NULL)
-        {
-            xkb->settings->kbd_config = g_new0 (t_xkb_kbd_config, 1);
-        }
-        xkb->settings->kbd_config->model = g_strdup (xfce_rc_read_entry (rcfile, "model", NULL));
-        xkb->settings->kbd_config->layouts = g_strdup (xfce_rc_read_entry (rcfile, "layouts", NULL));
-        xkb->settings->kbd_config->variants = g_strdup (xfce_rc_read_entry (rcfile, "variants", NULL));
-        xkb->settings->kbd_config->toggle_option = g_strdup (xfce_rc_read_entry (rcfile, "toggle_option", NULL));
-        xkb->settings->kbd_config->compose_key_position = g_strdup (xfce_rc_read_entry (rcfile, "compose_key_position", NULL));
+        g_assert (xkb->settings->kbd_config != NULL);
+        xkb->settings->kbd_config->model = g_strdup (xfce_rc_read_entry (rcfile, "model", xkb->settings->kbd_config->model));
+        xkb->settings->kbd_config->layouts = g_strdup (xfce_rc_read_entry (rcfile, "layouts", xkb->settings->kbd_config->layouts));
+        xkb->settings->kbd_config->variants = g_strdup (xfce_rc_read_entry (rcfile, "variants", xkb->settings->kbd_config->variants));
+        xkb->settings->kbd_config->toggle_option = g_strdup (xfce_rc_read_entry (rcfile, "toggle_option", xkb->settings->kbd_config->toggle_option));
+        xkb->settings->kbd_config->compose_key_position = g_strdup (xfce_rc_read_entry (rcfile, "compose_key_position", xkb->settings->kbd_config->compose_key_position));
 
         xfce_rc_close (rcfile);
 
@@ -326,7 +335,7 @@ static void
 xkb_load_default (t_xkb *xkb)
 {
     xkb->display_type = DISPLAY_TYPE_IMAGE;
-    xkb->display_textsize = DISPLAY_TEXTSIZE_SMALL;
+    xkb->display_textsize = DISPLAY_TEXTSIZE_LARGE;
     xkb->settings->group_policy = GROUP_POLICY_PER_APPLICATION;
     xkb->settings->default_group = 0;
     xkb->settings->kbd_config = NULL;
@@ -335,17 +344,39 @@ xkb_load_default (t_xkb *xkb)
 static gboolean
 xkb_calculate_sizes (t_xkb *xkb, GtkOrientation orientation, gint panel_size)
 {
+    guint nrows = 1;
+
+#ifdef HAS_PANEL_49
+    nrows       = xfce_panel_plugin_get_nrows (xkb->plugin);
+    panel_size /= nrows;
+#endif
+    TRACE ("calculate_sizes(%p: %d,%d)", xkb, panel_size, nrows);
 
     switch (orientation)
     {
         case GTK_ORIENTATION_HORIZONTAL:
             xkb->vsize = panel_size;
-            xkb->hsize = (int) (1.33 * panel_size);
+            if (nrows > 1)
+            {
+                xkb->hsize = xkb->vsize;
+            }
+            else
+            {
+                xkb->hsize = (int) (1.33 * panel_size);
+            }
+
             gtk_widget_set_size_request (xkb->btn, xkb->hsize, xkb->vsize);
             break;
         case GTK_ORIENTATION_VERTICAL:
             xkb->hsize = panel_size;
-            xkb->vsize = (int) (0.75 * panel_size);
+            if (nrows > 1)
+            {
+                xkb->vsize = xkb->hsize;
+            }
+            else
+            {
+                xkb->vsize = (int) (0.75 * panel_size);
+            }
             if (xkb->vsize < 10) xkb->vsize = 10;
             gtk_widget_set_size_request (xkb->btn, xkb->hsize, xkb->vsize);
             break;
@@ -353,6 +384,8 @@ xkb_calculate_sizes (t_xkb *xkb, GtkOrientation orientation, gint panel_size)
             break;
     }
 
+    DBG ("size requested: h/v (%p: %d/%d)", xkb, xkb->hsize, xkb->vsize);
+
     xkb_refresh_gui (xkb);
     return TRUE;
 }
@@ -420,8 +453,16 @@ xkb_initialize_menu (t_xkb *xkb)
 void
 xkb_refresh_gui (t_xkb *xkb)
 {
+    GdkDisplay * display;
+
     /* Part of the image may remain visible after display type change */
     gtk_widget_queue_draw_area (xkb->btn, 0, 0,
             xkb->button_hsize, xkb->button_vsize);
+
+    display = gdk_display_get_default();
+    if (display)
+    {
+        gtk_tooltip_trigger_tooltip_query(display);
+    }
 }
 
