@@ -23,6 +23,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "xfce4-xkb-plugin.h"
 #include "xkb-config.h"
 
 #include <stdio.h>
@@ -64,21 +65,26 @@ typedef struct
     XklConfigRec         *config_rec;
 } t_xkb_config;
 
-t_xkb_config *config;
+static t_xkb_config *config;
+static gboolean ignore_xkl_config_change = FALSE;
 
-void                xkb_config_state_changed            (XklEngine *engine,
-                                                         XklEngineStateChange *change,
+static void         xkb_config_state_changed            (XklEngine *engine,
+                                                         XklEngineStateChange change,
                                                          gint group,
-                                                         gboolean restore);
+                                                         gboolean restore,
+                                                         gpointer user_data);
 
-void                xkb_config_xkl_config_changed       (XklEngine *engine);
+static void         xkb_config_xkl_config_changed       (XklEngine *engine,
+                                                         gpointer user_data);
+void                xkb_config_reset_xkl_config         (XklEngine *engine,
+                                                         gpointer   user_data);
 
-GdkFilterReturn     handle_xevent                       (GdkXEvent * xev,
+static GdkFilterReturn  handle_xevent                   (GdkXEvent * xev,
                                                          GdkEvent * event);
 
-void                xkb_config_update_configuration     (t_xkb_settings *settings);
 static void         xkb_config_free                     ();
 static void         xkb_config_initialize_xkb_options   (t_xkb_settings *settings);
+static gboolean     xkb_config_activate_xkl_record      ();
 
 /* ---------------------- implementation ------------------------- */
 
@@ -117,6 +123,10 @@ xkb_config_initialize (t_xkb_settings *settings,
             "X-config-changed",
             G_CALLBACK (xkb_config_xkl_config_changed),
             NULL);
+    g_signal_connect (config->engine,
+            "X-new-device",
+            G_CALLBACK (xkb_config_reset_xkl_config),
+            NULL);
     gdk_window_add_filter (NULL, (GdkFilterFunc) handle_xevent, NULL);
 
     return TRUE;
@@ -293,6 +303,7 @@ xkb_config_update_settings (t_xkb_settings *settings)
 
     gchar **opt;
     gchar **prefix;
+    int i;
 
     g_assert (config != NULL);
     g_assert (settings != NULL);
@@ -306,7 +317,25 @@ xkb_config_update_settings (t_xkb_settings *settings)
 
     if (settings->kbd_config == NULL || settings->never_modify_config)
     {
-        xkl_config_rec_get_from_server (config->config_rec, config->engine);
+        xkl_config_rec_reset (config->config_rec);
+        if (!xkl_config_rec_get_from_server (config->config_rec, config->engine))
+        {
+            DBG ("ERROR: can't get xkl config: [%s]", xkl_get_last_error());
+        }
+
+        /* XklConfigRec uses for NULL for empty variant instead of "".
+           So if has skipped variants we can't get proper settings->kbd_config->variants.
+           xkl will also get confused if we try to activate() any config_rec
+           that has NULLs for empty variants.
+           Hence, we'll replace NULL variants with ""
+        */
+        for (i = 0; config->config_rec->layouts[i]; i++)
+        {
+            if (!config->config_rec->variants[i])
+                config->config_rec->variants[i] = g_strdup ("");
+        }
+
+        XKB_DEBUG_CONFIG_REC (config->config_rec, "retrieved");
         if (settings->kbd_config == NULL)
             settings->kbd_config = g_new0 (t_xkb_kbd_config, 1);
 
@@ -314,27 +343,8 @@ xkb_config_update_settings (t_xkb_settings *settings)
         settings->kbd_config->model = g_strdup (config->config_rec->model);
         g_free (settings->kbd_config->layouts);
         settings->kbd_config->layouts = g_strjoinv (",", config->config_rec->layouts);
-
-        /* XklConfigRec uses for NULL for empty variant instead of "".
-         * So if has skipped variants we can't get proper settings->kbd_config->variants.
-         * So I use this hack to get proper kbd_config->variants */
-        gchar *tmp1 = g_strdup("");
-        gchar *tmp2 = NULL;
-        int i;
-        for (i = 0; config->config_rec->layouts[i]; i++)
-        {
-            tmp2 = g_strconcat (tmp1, config->config_rec->variants[i] ? config->config_rec->variants[i] : "", NULL);
-            g_free(tmp1);
-            tmp1 = tmp2;
-            if (config->config_rec->layouts[i + 1])
-            {
-                tmp2 = g_strconcat (tmp1, ",", NULL);
-                g_free(tmp1);
-                tmp1 = tmp2;
-            }
-        }
         g_free (settings->kbd_config->variants);
-        settings->kbd_config->variants = tmp2;
+        settings->kbd_config->variants = g_strjoinv (",", config->config_rec->variants);
     }
     else
     {
@@ -343,7 +353,7 @@ xkb_config_update_settings (t_xkb_settings *settings)
         activate_settings = TRUE;
 
         g_free (config->config_rec->model);
-        config->config_rec->model = g_strdup (settings->kbd_config->model);
+        config->config_rec->model = g_strdup (settings->kbd_config->model ? settings->kbd_config->model : "");
 
         g_strfreev (config->config_rec->layouts);
         config->config_rec->layouts = g_strsplit_set (settings->kbd_config->layouts, ",", 0);
@@ -393,7 +403,13 @@ xkb_config_update_settings (t_xkb_settings *settings)
     }
 
     if (activate_settings && !settings->never_modify_config)
-        xkl_config_rec_activate (config->config_rec, config->engine);
+    {
+        if (!xkb_config_activate_xkl_record ())
+        {
+            // FIXME: we should probably try to reload from xkl config here
+            return FALSE;
+        }
+    }
 
     xkb_config_initialize_xkb_options (settings);
 
@@ -407,7 +423,6 @@ xkb_config_window_changed (guint new_window_id, guint application_id)
     gpointer key, value;
     GHashTable *hashtable;
     guint id;
-    gint DEBUG_FOUND = 0;
 
     g_assert (config != NULL);
 
@@ -437,7 +452,6 @@ xkb_config_window_changed (guint new_window_id, guint application_id)
     if (g_hash_table_lookup_extended (hashtable, GINT_TO_POINTER (id), &key, &value))
     {
         group = GPOINTER_TO_INT (value);
-        DEBUG_FOUND = 1;
     }
 
     g_hash_table_insert (
@@ -530,9 +544,10 @@ xkb_config_get_variant (gint group)
 
 void
 xkb_config_state_changed (XklEngine *engine,
-                          XklEngineStateChange *change,
+                          XklEngineStateChange change,
                           gint group,
-                          gboolean restore)
+                          gboolean restore,
+                          gpointer user_data)
 {
     if (change == GROUP_CHANGED)
     {
@@ -563,14 +578,30 @@ xkb_config_state_changed (XklEngine *engine,
 }
 
 void
-xkb_config_xkl_config_changed (XklEngine *engine)
+xkb_config_xkl_config_changed (XklEngine *engine, gpointer user_data)
 {
+    TRACE ("ignore_xkl_config_change: %d ", ignore_xkl_config_change);
+
+    if (ignore_xkl_config_change) return;
+
     kbd_config_free (config->settings->kbd_config);
     config->settings->kbd_config = NULL;
     xkb_config_update_settings (config->settings);
 
     if (config->callback != NULL)
-        config->callback (xkb_config_get_current_group (), TRUE, config->callback_data);
+    {
+        xkb_config_set_group (0);
+        config->callback (0, TRUE, config->callback_data);
+    }
+}
+
+void
+xkb_config_reset_xkl_config (XklEngine *engine,
+                             gpointer   user_data)
+{
+    TRACE ("X-new-device: %d", config->settings->never_modify_config);
+    if (!config->settings->never_modify_config)
+        xkb_config_activate_xkl_record ();
 }
 
 gint
@@ -628,3 +659,26 @@ xkb_config_get_max_layout_number (void)
     return xkl_engine_get_max_num_groups (config->engine);
 }
 
+gboolean
+xkb_config_activate_xkl_record (void)
+{
+    gboolean rc;
+
+    if (config->config_rec->model == NULL)
+    {
+        config->config_rec->model = g_strdup ("");
+    }
+
+    XKB_DEBUG_CONFIG_REC (config->config_rec, "activate");
+
+    ignore_xkl_config_change = TRUE;
+    rc = xkl_config_rec_activate (config->config_rec, config->engine);
+    ignore_xkl_config_change = FALSE;
+
+    if (!rc)
+    {
+        DBG ("ERROR: can't activate xkl config: [%s]", xkl_get_last_error());
+    }
+
+    return rc;
+}
