@@ -39,9 +39,8 @@
 #include "xkb-settings-dialog.h"
 #include "xkb-util.h"
 
-GtkTreeIter current_iter;
-GtkWidget *settings_dialog;
-GtkWidget *default_layout_menu;
+static GtkTreeIter current_iter;
+static GtkWidget *settings_dialog;
 
 enum combo_enum
 {
@@ -304,7 +303,7 @@ xkb_settings_set_kbd_combo_default_value (t_xkb *xkb)
         return;
 
     gtk_tree_model_get (model, &iter, NOM, &id, -1);
-    if (strcmp (id, config->model) == 0 )
+    if (config->model && strcmp (id, config->model) == 0 )
         gtk_combo_box_set_active_iter (GTK_COMBO_BOX (xkb->kbd_model_combo), &iter);
     else
     {
@@ -313,7 +312,7 @@ xkb_settings_set_kbd_combo_default_value (t_xkb *xkb)
             g_free (id);
             gtk_tree_model_get (model, &iter, NOM, &id, -1);
 
-            if (strcmp (id, config->model) == 0)
+            if (config->model && strcmp (id, config->model) == 0)
             {
                 gtk_combo_box_set_active_iter (GTK_COMBO_BOX (xkb->kbd_model_combo), &iter);
                 break;
@@ -439,7 +438,7 @@ xkb_settings_rm_layout (GtkWidget *widget, t_xkb *xkb)
         {
             if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter))
             {
-                gtk_list_store_set (GTK_LIST_STORE (model), &iter, DEFAULT_LAYOUT, TRUE);
+                gtk_list_store_set (GTK_LIST_STORE (model), &iter, DEFAULT_LAYOUT, TRUE, -1);
             }
         }
         gtk_widget_set_sensitive (xkb->add_layout_btn, TRUE);
@@ -747,17 +746,22 @@ void
 xfce_xkb_about (XfcePanelPlugin *plugin)
 {
     GtkWidget *about;
-    const gchar* authors[3] = {
+    const gchar* authors[] = {
         "Alexander Iliev <sasoiliev@mamul.org>",
         "Gauvain Pocentek <gauvainpocentek@gmail.com>",
+        "Igor Slepchin <igor.slepchin@gmail.com>",
         NULL
     };
 
     about = gtk_about_dialog_new ();
     gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (about),
             _("Keyboard Layouts Plugin"));
+    gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (about),
+            PACKAGE_VERSION);
     gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG (about),
             NULL);
+    gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (about),
+            xfce_get_license_text (XFCE_LICENSE_TEXT_GPL));
     gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (about),
             (const gchar**) authors);
     gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (about),
